package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.Polygon;
import java.awt.Image;
import java.util.Random;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import javax.swing.JPanel;
public class Bonus extends Moving implements IDrawable
{
    private static final double size = 20; // taille du bonus du centre de gravit > un sommet (le plus loign).
    /* Variable relatif aux images : largeur, hauteur, ratio d'agrandissement de l'image */
    private static final int IWIDTH = 40, IHEIGHT = 60; 
    private static final double IRATE = ((size * 2) / IHEIGHT);
    
    private static final String[] BONUSTYPE = {"Bonus de vie", "Bonus de division", "Malus de division", "Bonus d'invulnrabilit", "Bonus radar", "Malus radar", "Malus teleportation", "Bonus temps", "Malus temprature", "Bonus temprature" };
    
    private Point2D.Double[] peaks; //on mmorise les 4 coins des bonus pour ne pas les recalculer pour chaque collision
    private final double TIMELIFE = 10000; // vitesse et temps de vie par dfaut
    
    private int type = 0;
    
    /**
    * Un bonus aura un temps de vie limit, poura se dplacer et tourner sur lui mme.
    * @param pPos position du bonus.
    * @param vD vecteur direction du bonus.
    */
    public Bonus(Point2D.Double pPos, Point2D.Double vD)
    {
        super(pPos, vD, Config.getBonusSpeed());
        Random rand = new Random();
        type = rand.nextInt(BONUSTYPE.length);
        setTimeToDead(TIMELIFE);       
        refreshPeaks(); // on gnre les 4 coins
    }
    
    /**
    * Un bonus aura un temps de vie limit, poura se dplacer et tourner sur lui mme.
    * @param pPos position du bonus.
    * @param t type du bonus.
    */
    public Bonus(Point2D.Double pPos, Point2D.Double vD, int t)
    {
        super(pPos, vD, Config.getBonusSpeed());
        Random rand = new Random();
        type = t;
        setTimeToDead(TIMELIFE);       
        refreshPeaks(); // on gnre les 4 coins
    }
    
    /**
    *   Mets  jours les coordonne du bonus.
    *   Les bonus s'loignent tous du vaisseau  la mme vitesse (qui varie en fct du level).
    */
    public void refreshPosition()
    {
        setPos(Geom.add(getPos(), Geom.normalize(Geom.vector(Config.getPCenter(), getPos()), Config.getRefresh() * Config.getCurrentSpeedFactor() * 1D/42) ) );  
        refreshPeaks(); // on gnre les 3 coins
    }
    
    
    /**
	 * @return les coordonnes des 4 sommets du bonus sous forme d'un polygon.
	 */
	public Polygon getPolygon()
    {
        Point2D.Double[] tab = getPeaks();
        int[] pointx = {(int) tab[0].getX(), (int) tab[1].getX(), (int) tab[2].getX(), (int) tab[3].getX()};
        int[]  pointy = {(int) tab[0].getY(), (int) tab[1].getY(), (int) tab[2].getY(), (int) tab[3].getY()};
        return new Polygon (pointx, pointy, 4);   
    }
    
    /**
    * @return une liste contenant les 4 sommets du bonus.
    */
    public Point2D.Double[] getPeaks()
    {
        return peaks;
    }  
    
    /**
    * Mets  jours la position des coins du vaisseau
    */
    public void refreshPeaks()
    {
        Point2D.Double p0 = Geom.translate(getPos(), Geom.normalize(getOrien(), size));
        Point2D.Double p1 = Geom.translate(getPos(), Geom.rotate(Geom.normalize(getOrien(), (2*size)/3), 90));
        Point2D.Double p2 = Geom.translate(getPos(), Geom.rotate(Geom.normalize(getOrien(), size), 180));
        Point2D.Double p3 = Geom.translate(getPos(), Geom.rotate(Geom.normalize(getOrien(), (2*size)/3), 270));
        Point2D.Double[]  tab = {p0, p1, p2, p3}; 
        peaks = tab;
    }
    
    /**
    * @return true si un point percute le bonus
    */
    public boolean isInterPoint(Point2D.Double p)
    {
        return getPolygon().contains(p);
    }  
  
    
    /**
    * @return un rayon pour approximer les collisions
    */
    public double getRadius()
    {
        return size;
    }
    
    /**
    *   @return le type de bonus
    *   0 un bonus vie
    *   1 un malus division
    *   2 un bonus invulnrabilit
    */
    public double getType()
    {
        return type;
    }
    
    /**
    * @return le type de bonus sous forme de texte.
    */
    public String toString()
    {
        return BONUSTYPE[type];
    }
    
    /**
    * @return le nombre de bonus diffrent implment.
    */
    public static int getNbBonus()
    
    {
        return BONUSTYPE.length;
    }
    /**
    * Gnre un bonus proche du vaisseau
    * @param t type du bonus.
    */
    public static Bonus generateBonus(int t)
    {
        Random rand = new Random();
        // la distance varie entre 0 et le rayon du cercle d'affichage
        int distance = rand.nextInt((int) (Config.getDistancePrintMobile()));
        Point2D.Double pos = Geom.randomPointEdgeCircle(Config.getPCenter(), distance);
        Point2D.Double vDirect = Geom.vectorRandom();
        return new Bonus(pos, vDirect, t);
    } 
    

      
    /**
     * Mthode pour dessiner un bonus.
     */
    public void draw(Graphics g)
    {
       /*   // dessine un Polygon bleu
        g.setColor(Color.blue);
        g.fillPolygon(getPolygon());  */
     
        Graphics2D g2d = (Graphics2D) g;
        AffineTransform at = new AffineTransform();
        Point2D.Double v = Geom.inverseVector(getOrien());
        // on calcule le point superieur gauche de l'image
        Point2D.Double pCorner = Geom.translate(Geom.translate(getPos(), Geom.normalize(getOrien(), size)), Geom.rotate(Geom.normalize(v, (2*size)/3), 90));
        // on place l'image
        at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
        // on la tourne
        at.rotate(Math.toRadians(Geom.angle(getOrien()) + 90));
        // on la reduit  la bonne taille
        at.scale(IRATE * Config.getFrameRate(), IRATE * Config.getFrameRate());
        g2d.drawImage(Engine.images.bonus[type], at, null);
        
        /* Pour ne pas avoir un jeu static en pause, on a dessid de tourner les bonus  chaque repaint */
        setOrien(Geom.rotate(getOrien(),Config.getTurnFactor()/5D)); 
    } 
}