package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.geom.Point2D;
import java.awt.geom.Ellipse2D;
import java.util.*;
import java.awt.Dimension;
public class Engine
{
    private LinkedList<Asteroid> linkedAsteroid;
    private ArrayList<Missile> arrayMissile;
    private ArrayList<Bonus> arrayBonus;
    private LinkedList<Burst> linkedBurst;
    private SpaceShip ship;
    private ArrayList<Astronaut> arrayAstronaut;
    private static Life life; // vie du vaisseau
    public final static ImageGame images = new ImageGame();
    private static Scoring score; // score de la partie
    private static Level level;
    private static Radar radar;
    private static PlayTime playTime; // mmorise le temps de la partie
    public static BackGround backGround = new BackGround(); // contien le fond "mobile" du jeu
    /* Variable pour voir si le jeu a dj tait lanc depuis l'ouverture de la fentre, si on a un game over ou si on est en pause */
    private static boolean breakGame = false;
    private static int nbGame = 0;
    private static boolean gameOver = true;

       
    /**
    * Construit un jeu
    *
    * @param frameX la largeur de la fentre.
    * @param frameY la hauteur de la fentre.
    */
    public Engine(int frameX, int frameY)
    {
        Config.setFrame(frameX, frameY);
        linkedAsteroid = new LinkedList<Asteroid>();
        arrayMissile = new ArrayList<Missile>(0);
        arrayBonus = new ArrayList<Bonus>(0);
        arrayAstronaut = new ArrayList<Astronaut>(0);
        linkedBurst = new LinkedList<Burst>();
                
        
        playTime = new PlayTime();
        ship = new SpaceShip(); // initialise le vaisseau
        score = new Scoring();
        level = new Level(arrayAstronaut);
        radar = new Radar(arrayAstronaut);
        life = new Life();
        backGround = new BackGround();
        

        // On va nettoyer l'cran        
        Display.clearIDrawable(0);
        Display.clearIDrawable(1);
        Display.clearIDrawable(2);
        Display.clearIDrawable(3);
        Display.clearIDrawable(4);
        Display.clearIDrawable(5);
        
        // on rajoute les lments  l'affichage
        Display.addIDrawable(backGround, 0);    
        // on rajoute l'onde du radar
        Display.addIDrawable(radar.getRadarWave(), 0);
        Display.addIDrawable(GuiEvent.getBackGroundGradation(), 2);
        Display.addIDrawable(ship, 2);
        Display.addIDrawable(playTime, 4);
        Display.addIDrawable(level, 4);
        Display.addIDrawable(score, 4);
        Display.addIDrawable(life, 4);
        Display.addIDrawable(radar, 4);
        
        breakGame = false;
        nbGame += 1;
        gameOver = false;
        new Key(); // On rinitialise le keylistener         
        
    }
     
    
    /**
    * Ajout un astroide dans le jeu  une position alatoire.
    */
    public void addRandomAsteroid()
    {
        linkedAsteroid.addLast(Asteroid.randomAsteroid());
    }
    
    /**
    * Fonction permetant d'avoir un nombre minimum d'astrodes dans le jeu
    */
    public void minAsteroid()
    {
        if (linkedAsteroid.size() < Config.getAsteroidNb())
        {
            for ( int i = linkedAsteroid.size() - 1; i < Config.getAsteroidNb(); i++)
            {
                addRandomAsteroid();
            }
        }
    }
    
    /**
    * Mets  jour la position des astroides, des missiles du vaisseau et des bonus.
    */
    public void refreshPosition()
    {
        /* Mets  jours le vaisseau */
        ship.centerPosition(); //centre le vaisseau
        ship.refresh(MouseMotion.getMousePointer()); // on oriente le canon vers la souris
        
        /* Mets  jours les asteroides */
        for (Asteroid aste : linkedAsteroid)
        {
            aste.refreshPosition();
            //deplace l'asteroide en fct du vaisseau           
            aste.translate(ship.getTranslate());
        } 
        
        /* Mets  jours les missiles */
        // pas d'itrateur car modification de la liste
        for (int i = 0; i < arrayMissile.size(); i++)
        {
            arrayMissile.get(i).refreshPosition(); 
            //deplace le missile en fct du vaisseau           
            arrayMissile.get(i).translate(ship.getTranslate());
            if (arrayMissile.get(i).isDead())
            {   
                // si le missile est trop vieux il explose et disparait
                linkedBurst.add(new Burst(arrayMissile.get(i).getPos(), arrayMissile.get(i).getSize() * 1.5));
                arrayMissile.remove(arrayMissile.get(i));
            }
        }    
        
        /* Mets  jours les bonus */
        for (int i = 0; i < arrayBonus.size(); i++)
        {
            arrayBonus.get(i).refreshPosition();
            //dplace le missile en fct du vaisseau           
            arrayBonus.get(i).translate(ship.getTranslate());
            if (arrayBonus.get(i).isDead())
            {
                arrayBonus.remove(arrayBonus.get(i));
            }
        }   
        
        /* Mets  jours les explosions */
        for (int i = 0; i < linkedBurst.size(); i++)
        {
            linkedBurst.get(i).refreshPosition();
            //dplace le missile en fct du vaisseau           
            linkedBurst.get(i).translate(ship.getTranslate());
            if (linkedBurst.get(i).isDead())
            {
                linkedBurst.remove(linkedBurst.get(i));
            }
        }        
        
        /* Mets  jours les astronautes */
        for (Astronaut astro : arrayAstronaut)
        {
            astro.refreshPosition();
            //deplace le missile en fct du vaisseau           
            astro.translate(ship.getTranslate());
        }
        
        /* Mets  jours la position du fond */
        backGround.translate(ship.getTranslate());
    }
    
    /**
    * Permet de gnr un bonus (celon une proportion dfinie par Config).
    */
    public void winBonus(Point2D.Double pos, Point2D.Double direct)
    {
        if(Math.random() < Config.getBonusFactor() )
        {
            arrayBonus.add( new Bonus(pos, direct));
        }    
    }

    /**
    * Mets  jour la liste d'astroide et de missiles en cas de collision avec entre un missile et un asteroide.
    */
    public void collMissileAsteroid()
    {
        int jMissilleMax = arrayMissile.size();
        int iAsteroidMax = linkedAsteroid.size(); //on utilise une variable car en cas de division, la liste est rduite de un (on ne regarde pas les nouveaux asteroides  ce tic)
        int j = 0;
        while (j < jMissilleMax)
        {
            int i = 0;
            boolean collisionIlYA = false;
            while ( i < iAsteroidMax && !collisionIlYA)
            {
                if (Geom.isInterCiclePoint(linkedAsteroid.get(i).getPos(), linkedAsteroid.get(i).getRadius(), arrayMissile.get(j).getPos()))
                {
                //on a une collision
                    collisionIlYA = true;
                    divAsteroid(linkedAsteroid.get(i), arrayMissile.get(j).getPos(), Geom.normalize(arrayMissile.get(j).getDirect(), arrayMissile.get(j).getSpeed()), true);
                    iAsteroidMax --;
                }
                i++;
            }
            if (collisionIlYA) 
            {       
                arrayMissile.remove(arrayMissile.get(j));
                jMissilleMax--;
            }
            else
            {
                j++;
            }
        } 
    }
    
    /**
    * Mets  jour la liste des astroids
    * @return true si l'astroids tait divisible, false sinon
    * @param asteroid un astroide
    * @param bonus si true on gagne du score et u bonus peut tre gnr
    * @param pColl un point de collision
    * @param vColl un vecteur vitesse de l'objet qui rente en collision
    */
    public boolean divAsteroid(Asteroid asteroid, Point2D.Double pColl, Point2D.Double vColl, boolean bonus)
    {
        //On ajoute l'explosion 1.5* plus grande que le rayon de l'astroide
        linkedBurst.add(new Burst(asteroid.getPos(), asteroid.getRadius() * 1.5));
        if (asteroid.isDivisible())
        { //on divise l'astroide
            // Point de gnration des astroides enfants
            Point2D.Double pGen = asteroid.getPos();
            //Si on a un point de collision on le palce entre le point de collision et le centre de l'astroide
            if (pColl != null)
            {
                //vecteur du centre de l'astroides au point de collision
                Point2D.Double vPosColl = Geom.vector(pGen, pColl);
                vPosColl = Geom.normalize(vPosColl, Geom.magnitude(vPosColl) / 2D);
                pGen = Geom.add(pGen, vPosColl);
            }
            /* 
                On choisi la direction futur des astroides enfants
                Ainsi que leur vitesse
            */
            Point2D.Double vDirect = asteroid.getDirect();
            double speed = asteroid.getSpeed() / Config.getCurrentAsteroidRadiusDivFactor();
            if (vColl != null)
            {
                // on va addition le vecteur directeur de l'astroide * sa vitesse avec le vecteur vitesse entre en paramettre 
                vDirect = Geom.add(Geom.normalize(vDirect, speed), vColl);
                speed = Geom.magnitude(vDirect);
            }
            
            // On calcule le rayon des astroides enfants
            double radius = asteroid.getRadius() / Config.getCurrentAsteroidRadiusDivFactor();
            
            /* 
                On choisi la position des 2 astroides enfant en fct des donnes calcules plus haut
                A savoir le point de gnration + (la direction * radius + la direction * radius tourn de 90/-90 ) 
            */
            Point2D.Double vDeplacement = Geom.normalize(vDirect, radius);
            //Point 1 90
            Point2D.Double vDeplacementRadius = Geom.rotate(vDeplacement, 90);
            Point2D.Double p1 = Geom.add(pGen, Geom.add(vDeplacement, vDeplacementRadius));
            // Point 2 -90
            vDeplacementRadius = Geom.rotate(vDeplacement, -90);
            Point2D.Double p2 = Geom.add(pGen, Geom.add(vDeplacement, vDeplacementRadius));
            
            /*
                On recalcul vDirect (oui je sais a fait bcp de calcule)
                qui sera le vecteur entre pGen et p1/p2 
                On ne le normalise pas car le longeur du vecteur ne rentre pas en compte pour le dplacement de l'astroide
            */    
            
            linkedAsteroid.addLast(new Asteroid(p1, Geom.vector(pGen, p1),(int) (speed), radius));            
            linkedAsteroid.addLast(new Asteroid(p2, Geom.vector(pGen, p2),(int) (speed), radius));
        }
        
        //On ajout le score et on a une chance d'avoir un bonus si bonus true
        if (bonus)
        {
            score.destructionAsteroid(asteroid.getLifeTime(), asteroid.getRadius());
            winBonus(asteroid.getPos(), asteroid.getDirect());

        }
        //on retire l'astroide
        linkedAsteroid.remove(asteroid); 
        return asteroid.isDivisible();
    }
    
    /**
    * Mets  jours la liste des asteroides aprs la division de tout ceux si.
    * @param bonus permet d'activer ou non le bonus score
    */
    public void divAsteroidAll(boolean bonus)
    {
        /* On va parcourir le premier astroide size fois ( chaque parcours il sera supprim par divAsteroid() */
        int size = linkedAsteroid.size();
        for (int i = 0; i < size; i++)
        {
            divAsteroid(linkedAsteroid.getFirst(), null, null, bonus);
        }
    }
    
    /**
    * Mets  jour la liste des bonus en cas de collision avec entre un bonus et le vaisseau.
    */
    public void collBonusShip()
    {
        int i = 0;
        while (i < arrayBonus.size())
        {
            //on verifie s'il y a collion
            if (collBonusShip(ship, arrayBonus.get(i)) )
            {
                if (arrayBonus.get(i).getType() == 0)
                {
                    //bonus vie
                    life.increaseLife(Config.getBonusLife());
                }
                else if (arrayBonus.get(i).getType() == 1)
                {
                    // bonus division
                    divAsteroidAll(true);
                }
                else if (arrayBonus.get(i).getType() == 2)
                {
                    // malus division
                    divAsteroidAll(false);
                }
                else if (arrayBonus.get(i).getType() == 3)
                {
                    //bonus invulnerabilit
                    life.setInvulnerabilite(Config.getInvulnerableTime());
                }
                else if(arrayBonus.get(i).getType() == 4)
                {
                    //bonus radar
                    radar.setTImpulstionBonusEnd(Config.getDifficultiBonusRadar());
                }
                else if(arrayBonus.get(i).getType() == 5)
                {
                    //malus radar
                    radar.setTImpulsionBreackEnd(Config.getDifficultiMalusRadar());
                }
                else if(arrayBonus.get(i).getType() == 6)
                {
                    //bonus teleportation
                    //On gnre un vecteur d'orientation alatoir et de longeur alatoire
                    ship.setTranslate(Geom.normalize(Geom.vectorRandom(), Math.random() * Config.getDistanceMaxShip()));
                    //On est invulnerable pendant 1/3 du bonus invulnrabilit
                    life.setInvulnerabilite(Config.getInvulnerableTime() / 3);
                }
                else if(arrayBonus.get(i).getType() == 7)
                {
                    //bonus temps
                    level.setTimeEnd(Config.getDifficultiBonusTime());
                }
                else if(arrayBonus.get(i).getType() == 8)
                {
                    //Malus temprature
                    ship.setMalusTemp();
                }
                else
                {
                    //Bonus temprature
                    ship.setBonusTemp();
                }
                //on supprime le bonus
                arrayBonus.remove(arrayBonus.get(i));
            }    
            else
            {
                i++;
            }
        }
    }    
    
    /**
    * @return si le vaisseau est en collision avec le bonus.
    * @param s un vaisseau.
    * @param b un bonus.
    */
    public boolean collBonusShip(SpaceShip s, Bonus b)
    {
        /* On va regarder si la distance entre le vaisseau et le bonus est asses petites pour qu'il y ait collision, ensuite on va regarder si un des point du bonus est dans le vaisseau ou inversement */
        Point2D.Double[] peaksSpaceShip = s.getPeaks(); // 3 points
        Point2D.Double[] peaksBonus = b.getPeaks(); // 4 points
        return ( Geom.distance(s.getPos(), b.getPos()) <= (s.getRadius() + b.getRadius()) ) && ( s.isInterPoint(peaksBonus[0]) || s.isInterPoint(peaksBonus[1]) || s.isInterPoint(peaksBonus[2]) || s.isInterPoint(peaksBonus[3]) || b.isInterPoint(peaksSpaceShip[0]) || b.isInterPoint(peaksSpaceShip[1]) || b.isInterPoint(peaksSpaceShip[2]) );
    }
    
    /**
     * @return un arrayList de bonus.
     */
    public ArrayList<Bonus> getArrayBonus()
    {
        return arrayBonus;
    }    
    
    /**
     * @return un arrayList de astronautes.
     */
    public ArrayList<Astronaut> getArrayAstronaut()
    {
        return arrayAstronaut;
    }  
    
    /**
    * Vrifie si le vaisseau se prend un asteroide.
    */
    public void collAsteroidShip()
    {
        int i = 0;
        boolean collisionIlYA = false;
        while (i < linkedAsteroid.size() && !collisionIlYA)
        {
            if (ship.isInterCircle(linkedAsteroid.get(i).getPos(), linkedAsteroid.get(i).getRadius()) )
            {
                collisionIlYA = true;
            }
            i++;
        }
        if (collisionIlYA)
        {
            i--;
            if (!life.isInvulnerable())
            {
                // si on est pas invulnerable on perd de la vie
                life.increaseLife( - Config.getCollLife()); 
                // on devient invulnrable pour quelques secondes
                life.setInvulnerabilite(Config.getInvulnerableTime() / 3);

            }
            
            //On divise l'astroid (on ne fournis pas de point de collision)
            divAsteroid(linkedAsteroid.get(i), null, Geom.normalize(ship.getDirect(), ship.getSpeed()) , false);
            
        }
    }
    
    /**
    * Vrifie si on est proche d'un astronautes
    * Si oui on augmente sa barre de sauvetage
    * Si cette barre est remplie et qu'on est asss proche on le capture
    */
    public void collAstronautShip()
    {
        int i = 0, size = arrayAstronaut.size();
        while (i < size)
        {
            if (ship.isInterCircle(arrayAstronaut.get(i).getPos(), arrayAstronaut.get(i).getRadius()) )
            {
                // on survole l'astronaute 
                arrayAstronaut.get(i).decreaseTimeRescue(Config.getRefresh());
                if (arrayAstronaut.get(i).isSaved()) // on peut capturer l'astronaute
                {
                    level.increaseNbSpaceManCurrent();
                    score.destructionAstronaut();
                    //linkedAsteroid.remove(linkedAsteroid.get(i));
                    arrayAstronaut.remove(arrayAstronaut.get(i));
                    i--;
                    size--;
                }
            }            
            else if (ship.isInterCircle(arrayAstronaut.get(i).getPos(), arrayAstronaut.get(i).getRadius() * 2) )
            {
                arrayAstronaut.get(i).decreaseTimeRescue(Config.getRefresh() / 2);
            }
            i++;
        }
    }
    /**
    * Calcule les collision entre asteroides
    */
    public void collAsteroidAsteroid()
    {
        /* On va parcourir tout asterodes en boucle "triangulaire" pour ne pas tester la collision de 2 mme astrodes 2 fois */
        for (int i = 0; i < linkedAsteroid.size(); i++)
        {
            // on vrrifie si l'astrode n'a pas dj eu une collision rcement
            if (linkedAsteroid.get(i).canBeCollWithAsteroid())
            {
                for (int j = i +1; j < linkedAsteroid.size(); j++)
                {
                    if ( Geom.isInterTwoCircles(linkedAsteroid.get(i).getPos(), linkedAsteroid.get(i).getRadius(), linkedAsteroid.get(j).getPos(), linkedAsteroid.get(j).getRadius()) && linkedAsteroid.get(j).canBeCollWithAsteroid() )
                    {
                        /* Il y a collision 
                        Principe on inverse la direction des astroides, 
                        On inverse les vitesses mais avec une proportion en fct du rayon des astroides
                        On loigne les astroides de r1+r2 pour ne plus avoir de collision au tic suivant 
                        On signale que les 2 astroides sont rentr en collision
                        */
                        
                        // on inverse la vitesse et la direction des 2 asteroides
                        Point2D.Double vDirectTemps = linkedAsteroid.get(i).getDirect();
                        double speedTemps = linkedAsteroid.get(i).getSpeed();
                        linkedAsteroid.get(i).setDirect(linkedAsteroid.get(j).getDirect());
                        linkedAsteroid.get(i).setSpeed( linkedAsteroid.get(j).getSpeed() * (linkedAsteroid.get(j).getRadius() / linkedAsteroid.get(i).getRadius()) );
                        linkedAsteroid.get(j).setDirect(vDirectTemps);
                        linkedAsteroid.get(j).setSpeed(speedTemps * (linkedAsteroid.get(i).getRadius() / linkedAsteroid.get(j).getRadius()));
                        
                        //on va faire en sorte que les 2 asteroides soit  une distance r1+r2 pour ne pas avoir des collisions en cascade
                        linkedAsteroid.get(j).setPos(Geom.add(linkedAsteroid.get(i).getPos(), Geom.normalize(Geom.vector(linkedAsteroid.get(i).getPos(),linkedAsteroid.get(j).getPos()), (linkedAsteroid.get(i).getRadius() + linkedAsteroid.get(j).getRadius()) ) ));
                        
                        //on dit qu'il y a eu une collision  ce tic 
                        linkedAsteroid.get(i).setCollWithAsteroid();
                        linkedAsteroid.get(j).setCollWithAsteroid();
                    }
                }
            }
        }
    }
    
    /**
    * Lance la simulation
    */
    public void go()
    {
        /*On recupre les info rentr au clavier */
        if (Key.isKeyQPressed())  // si on appuie sur q on tourne  gauche 
            ship.turnLeft();
        if (Key.isKeyDPressed())  //si on appuie sur d on tourne  droite
            ship.turnRight();
            
        if (Key.isKeyZPressed()) // on appuie sur Z, on acclre et on augemente sa vitesse
        {
            ship.increaseAcceleration();
            ship.increaseSpeed();
        }
        else // on appuie pas sur z, on dcllre et on perd de la vitesse
        {
            ship.decreaseAcceleration(); 
            ship.decreaseSpeed();
        }
        
        if (Key.isKeySPressed()) // on appuie sur S on ralenti plus vite
        {
            ship.decreaseAcceleration();
            ship.decreaseSpeed();
        }
        //tir de missile
        // si on appuie sur espace ou sur le bouton gauche de la souris
        if(Key.isKeySpacePressed() || MouseKey.isKeyPressed())
        {
            if (ship.canFire())
            {
                arrayMissile.add(ship.fire());
            }
        }
        
        refreshPosition(); // on mets  jours les positions et supprimes les missiles si trop vieux
        radar.refresh();
        // ajoute le temps d'un tic  la partie
        playTime.increase(Config.getRefresh());

        minAsteroid(); 
        
        /* Gestion des collisions */
        
        collMissileAsteroid();
        collAsteroidAsteroid();
        collAsteroidShip();
        collBonusShip();
        collAstronautShip();
        
        
        life.refresh(); // mets  jours la vie affiche par le quadrant
        
        //on vrifie si le vaisseau  encrore de la vie
        if (life.isDead())
        {
            gameOver = true;
            Display.clearIDrawable(5);
            Display.addIDrawable(GuiEvent.getGameOver(), 5);
        }
        level.refresh(arrayAstronaut); // on rafraichit le lvl courrant

        /* On s'occupe de mettre  jour la liste des IDrawables a afficher  l'cran. */
        //on retire les bonus, missiles, astronautes, bonus de l'affichage
        Main.getDisplay().clearIDrawable(1);
        Main.getDisplay().clearIDrawable(3);
        
        
        // On ajout les astronautes.
        for (Astronaut astronaut : arrayAstronaut)
        {
            if (astronaut.isIDrawable())
            {
                Main.getDisplay().addIDrawable(astronaut, 1);
            }
        }
        
        // on ajoute les asteroides
        for (Asteroid asteroid : linkedAsteroid)
        {
            if (asteroid.isIDrawable())
            {
                Main.getDisplay().addIDrawable(asteroid, 1);
            }
        } 

        //on ajoute les missile.
        for(Missile missile : arrayMissile)
        {
            Main.getDisplay().addIDrawable(missile, 1);
            //si le missile est asss proche du vaisseau on l'ajoute  une couche superieur de l'affichage
            if (Geom.distance(Config.getPCenter(), missile.getPos()) <= ship.getRadius() + missile.getSize())
                Main.getDisplay().addIDrawable(missile, 3);
        }
        
        //on ajoute les bonus.
        for(Bonus bonus : arrayBonus)
        {
            Main.getDisplay().addIDrawable(bonus, 1);
        }    
        
        // On ajout les explosions.
        for (Burst burst : linkedBurst)
        {
            if (burst.isIDrawable())
            {
                Main.getDisplay().addIDrawable(burst, 1);
            }
        }
    }
        
    
    /**
     * @return un arrayList de Missile
     */
    public ArrayList<Missile> getArrayMissile()
    {
        return arrayMissile;
    }
     
    /**
     * @return un arrayList d'Asteroid.
     */
    public LinkedList<Asteroid> getArrayAsteroid()
    {
        return linkedAsteroid;
    }
    
    /**
     * @return SpaceShip un vaisseau.
     */
    public SpaceShip getShip()
    {
        return ship;
    }
       
    
    /**
    * @return l'objet life de la partie pour l'affichage.
    */
    public Life getLife()
    {
        return life;
    }
    
    /**
     * @return l'objet Score de la partie pour l'affichage.
     */
    public static Scoring getScore()
    {
        return score;
    }
    
    /**
    * @return l'objet mmorisant le temps de la partie
    */
    public static PlayTime getTPlay()
    {
        return playTime;
    }
    
    /**
    * @return si on a dj jou
    */
    public static boolean isAlreadyPlaying()
    {
        return nbGame > 0;
    }
    
    
    
    /**
     * Permet changer l'tat de pause du jeu.
     */
    public static void changeBreackGame()
    {
        SetBreackGame(!breakGame);
    }
    
    /**
     * Permet de mettre le jeu en pause / le reprendre
     * @param cond la condition pause si true, pas pause sinon
     */
    public static void SetBreackGame(boolean cond)
    {
        if (cond && nbGame > 0 && !gameOver)
        {
            breakGame = true; // on change l'tat de la pause
            Display.addIDrawable(GuiEvent.getBreackGame(), 5);
        }
        else if (nbGame > 0 && !gameOver)
        {
            breakGame = false; // on change l'tat de la pause
            Display.clearIDrawable(5);
        }
    }
    /**
     * Permet de mettre le jeu en pause.
     */
    public static void SetBreackGame()
    {
        SetBreackGame(true);
    }
    
    /**
     * @return si le jeu est en pause
     */
    public static boolean isBreackGame()
    {
        return breakGame;
    }
    
    
    /**
     * @return si le jeu a t perdu ou pas.
     */
    public static boolean isGameOver()
    {
        return gameOver;
    }
    
    /**
     * Permet de mettre le jeu en gameOver
     */
    public static void setGameOver(boolean bool)
    {
        gameOver = bool;
    }
    
    /**
    * @return l'objet level.
    */
    public static Level getLevel()
    {
        return level;
    }
    
    
    
}