package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.util.Random;
public class Moving
{
    private Point2D.Double pPos = new Point2D.Double(0,0); //position
    private Point2D.Double vOrient = new Point2D.Double(0, -1); // orientation
    private Point2D.Double vDirect = new Point2D.Double(0, 1); // direction
    private double speed = 0; // vitesse le mobile avant de speed pixel en 10 secondes.
    private double initTime = 0; //temps de vie de l'objet.
    private double timeDead; //le moment de destruction du mobile (en millisecondes) (sera dtruit une fois atteint).
     
    /**
    * @param pPos Position du mobile.
    */
    public Moving(Point2D.Double pPos)
    {
        this.pPos = pPos;
        initTime = PlayTime.getTime();
    }    
    
    /**
    * @param pPos Position du mobile.
    */
    public Moving(Point2D.Double pPos, double t)
    {
        this.pPos = pPos;
        timeDead = t + PlayTime.getTime();
        initTime = PlayTime.getTime();
    }
    
    /**
    *   Construit un objet mobile. Avec comme orientation la direction.
    * @param pPos Position du mobile.
    * @param vDirect Direction du mobile.
    * @param speed Vitesse du mobile.
    */
	public Moving(Point2D.Double pPos, Point2D.Double vDirect, double speed)
    {
        this.pPos = pPos;
        this.vOrient = vDirect;
        this.vDirect = vDirect;
        this.speed = speed;
        initTime = PlayTime.getTime();

    }

    /**
    *   Construit un objet mobile.
    * @param pPos Position du mobile.
    * @param vOrient Orientation du mobile.
    * @param vDirect Direction du mobile.
    * @param speed Vitesse du mobile.
    */
	public Moving(Point2D.Double pPos, Point2D.Double vOrient, Point2D.Double vDirect, double speed)
    {
        this.pPos = pPos;
        this.vOrient = vOrient;
        this.vDirect = vDirect;
        this.speed = speed;
        initTime = PlayTime.getTime();

    }    

    
    /**
    *   Mets  jours les coordonne du mobile en fonction du temps
    *   Avec modulo
    */
    public void refreshPosition()
    {
        refreshPosition(true);
    }    
    
    /**
    *   Mets  jours les coordonne du mobile en fonction du temps
    *   @param modulo si vrai onf ait un modulo si faut on en fait pas.
    */
    public void refreshPosition(boolean modulo)
    {
        pPos = Geom.add(pPos, Geom.normalize(vDirect, (speed / 1000D) * Config.getRefresh() * Config.getCurrentSpeedFactor()) );
        if (modulo)
            frameModulo();
    }
    
	/**
	 * Change la position (centre de gravit) du mobile.
	 * @param newPos La nouvelle position du centre de gravit
	 */
	public void setPos(Point2D.Double newPos)
    {
        pPos = newPos;
    }
    
   /**
    * @return la position du mobile (centre de gravit).
    */
    public Point2D.Double getPos()
    {
        return (Point2D.Double) pPos.clone() ;
    }

	/**
	 * Change l'orientation du mobile.
	 * @param newOrientation La nouvelle orientation.
	 */
	public void setOrien(Point2D.Double newOrientation)
    {
        vOrient = newOrientation;
    }
	
	/**
	 * 
	 * @return l'orientation du mobile
	 */
	public Point2D.Double getOrien()
    {
        return (Point2D.Double) vOrient.clone();
    }
    
    /**
    * @return si l'objet est  dtruire (pour pouvoir le dtruire si non).
    */
    public boolean isDead()
    {
        return (timeDead - PlayTime.getTime()) <= 0;
    }
    
	/**
	 * Change la direction (vecteur directeur) du mobile.
	 * @param newDirection le nouveau vecteur directeur.
	 */
	public void setDirect(Point2D.Double newDirection)
    {
        vDirect = newDirection;
    }
	
	/**
     * 
     * @return la direction (vecteur directeur) du mobile.
     */
	public Point2D.Double getDirect()
    {
        return (Point2D.Double) vDirect.clone();
    }

	/**
	 * Change la vitesse du mobile.
	 * @param newSpeed la nouvelle vitesse
	 */
	public void setSpeed(double newSpeed)
    {
        speed = newSpeed;
    }

	/**
	 * 
	 * @return la vitesse du mobile.
	 */
	public double getSpeed()
    {
        return speed;
    }


    /**
    * @return le temps de vie courrant de l'objet.
    */
    public double getLifeTime()
    {
        return PlayTime.getTime() - initTime;
    }
    
    /**
    * Modifie la valeur o le mobile sera considr comme mort.
    */
    public void setTimeToDead(double t)
    {
        timeDead = PlayTime.getTime() + t;
    }

    /**
    * Permet de faire rapararaitre  un nouvelle endroit si il est trop loin du vaisseau
    */
    public void frameModulo()
    {
        if (Geom.distance(Config.getPCenter(), getPos()) >= Config.getDistanceMaxShip())
        {
            // fait une symtrie centrale de la position du mobile par apport au vaisseau avec une distance gnre alatoirement
            Point2D.Double position = Geom.add(Config.getPCenter(), Geom.normalize(Geom.vector(getPos(), Config.getPCenter()), generateDistanceAdd() ));;
            setPos(position);
            // on redirige le mobile vers le vaisseau
            setDirect(generateDirection(position));
        }
    }    
    
    /**
    * Mets  jours la position du mobile en fonction d'un vecteur externe
    * @param vector le vecteur ncessaire  la translation.
    */
    public void translate(Point2D.Double vector)
    {
        setPos(Geom.add(Geom.inverseVector(vector), getPos()));
    }
    
    /**
    * @return un point allatoirement gnr dans l'espace non visible par le joueur
    */
    public static Point2D.Double generatePosition()
    {

        return Geom.randomPointEdgeCircle(Config.getPCenter(), generateDistanceAdd());
    }
    
    /**
    * @param p une position.
    * @return un vecteur direction orienter allatoirement vers le vaisseau (plus on est haut niveau plus il est dirig vers le vaisseau)
    */
    public static Point2D.Double generateDirection(Point2D.Double p)
    {
        Random rand = new Random();
        return Geom.normalize(Geom.rotate(Geom.vector(p, Config.getPCenter()), (rand.nextInt(Config.getMovingAddAngle() * 2) - Config.getMovingAddAngle() )) );
    }
    
    /**
    * @return une distance par apport au vaisseau pour l'ajout/modulo de mobile
    */
    public static double generateDistanceAdd()
    {
        Random rand = new Random();
        // la distance varie entre getDistanceDrawMobile et getDistanceMaxShip
        return rand.nextInt((int) (Config.getDistanceMaxShip() - Config.getDistanceDrawMobile())) + Config.getDistanceDrawMobile();
    }

    

}